using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using AGSS.Models;
using AGSS.Models.DTOs;
using AGSS.Models.Entities;
using AGSS.Models.Template;
using AGSS.Services;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;

namespace AGSS.Controllers.Admin;

/// <summary>
/// 控制器类，用于管理角色相关的操作，包括添加角色、分配角色给用户以及通过角色查询用户。
/// 该控制器仅限具有"Admin"角色的用户访问。
/// </summary>
[Authorize(Roles = "Admin")]
[Route("api/v1/[controller]")]
public class AdminRoleControllers:ControllerBase
{
    /// <summary>
    /// 用户服务实例，用于执行与用户相关的操作。
    /// 该服务提供了一系列方法来处理用户的查询和更新等操作，
    /// 包括但不限于获取用户详细信息、修改用户资料等功能。
    /// </summary>
    private readonly UserService _userService;

    public AdminRoleControllers(UserService userService, RoleManager<RoleModel> roleManager, UserManager<UserModel> userManager)
    {
        _userService = userService;
        _roleManager = roleManager;
        _userManager = userManager;
    }

    /// <summary>
    /// 角色管理器，用于处理角色相关的操作，如创建、查询等。
    /// 此角色管理器实例主要用于与RoleModel类型的实体进行交互，
    /// 支持添加新角色、为用户分配角色等功能。
    /// </summary>
    private readonly RoleManager<RoleModel> _roleManager;

    /// <summary>
    /// 用户管理器实例，用于处理用户相关的操作如添加角色、查询用户等。
    /// 此实例通过依赖注入的方式在构造函数中初始化，并在整个控制器生命周期内可用。
    /// </summary>
    private readonly UserManager<UserModel> _userManager; // Assuming UserModel is the type of user

    /// <summary>
    /// 管理员角色控制器，用于处理与角色相关的操作，如添加角色、分配角色给用户以及通过角色查询用户。
    /// 该控制器下的所有方法都需要管理员权限才能访问。
    /// </summary>


    /// <summary>
    /// 添加新角色
    /// </summary>
    /// <param name="role">要添加的角色信息</param>
    /// <returns>返回操作结果，包含状态码、消息和数据</returns>
    [HttpPost("role")]
    public async Task<IActionResult> AddRole(string rolename,string normalizedname)
    {
        
      

        var result = await _roleManager.CreateAsync(new RoleModel(){Id = Guid.Empty.ToString(),Name = rolename,NormalizedName = normalizedname});
        if (result.Succeeded)
        {
            return Ok(new ReturnTemplate(200,"创建成功",""));

        }
        else
        {
            return Ok(new ReturnTemplate(StatusCodes.Status500InternalServerError,"创建失败","Failed to create role: " + string.Join(", ", result.Errors.Select(e => e.Description))));
        }
    }

    /// <summary>
    /// 为指定用户分配角色
    /// </summary>
    /// <param name="userId">用户的唯一标识符</param>
    /// <param name="roleName">要分配的角色名称</param>
    /// <returns>返回一个包含操作结果的ReturnTemplate对象，其中Code表示状态码，Msg表示消息，Data表示附加数据（如果有的话）</returns>
    
    [HttpPost("role/endow")]
    public async Task<IActionResult> EndowRole(string userId, string roleName)
    {
        var user = await _userManager.FindByIdAsync(userId);
        if (user == null)
        {
            return Ok(new ReturnTemplate(400, "用户不存在", ""));
        }

        var role = await _roleManager.FindByNameAsync(roleName);
        if (role == null)
        {
            return Ok(new ReturnTemplate(400, "角色不存在", ""));
        }

        var result = await _userManager.AddToRoleAsync(user, role.Name);
        if (result.Succeeded)
        {
            return Ok(new ReturnTemplate(200, "现在该用户已经被赋予这个角色了", user));
        }
        else
        {
            return Ok(new ReturnTemplate(StatusCodes.Status500InternalServerError, "角色分配失败", "Failed to endow role: " + string.Join(", ", result.Errors.Select(e => e.Description))));
        }
    }


    /// <summary>
    /// 删除指定用户。
    /// </summary>
    /// <param name="userId">要删除的用户的唯一标识符。</param>
    /// <returns>返回操作结果，包含状态码、消息和数据。如果删除成功，则返回200状态码；如果用户ID为空或未找到指定用户，则分别返回400或404状态码；若删除过程中出现错误，则返回500状态码并附带错误信息。</returns>
    [HttpDelete("user")]
    public async Task<IActionResult> DelUser(string userId)
    {
        if (string.IsNullOrWhiteSpace(userId))
        {
            return Ok(new ReturnTemplate(400, "你打算拿个空的Id来骗我吗？", null));
        }

        var user = await _userManager.FindByIdAsync(userId);
        if (user == null)
        {
            return Ok(new ReturnTemplate(404, "你输了个假的用户吧......", null));
        }

        // 删除用户
        var result = await _userManager.DeleteAsync(user);
        if (result.Succeeded)
        {
            return Ok(new ReturnTemplate(200, "用户删除成功，不要留念这个用户哦~", null));
        }
        else
        {
            return StatusCode(500, new ReturnTemplate(500, "发生了一些不可预料的错误，555", result.Errors));
        }
    }
    
    
        [HttpPost("menu")]
        public async Task<IActionResult> SetMenu([FromBody]UserMenuRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.Id) || string.IsNullOrWhiteSpace(request.MenuName))
            {
                return Ok(new ReturnTemplate(400, "请求参数无效（有的参数是空的哦~）", ""));
            }
            var user=await _userManager.FindByIdAsync(request.Id);
            if (user==null)
            {
                return Ok(new ReturnTemplate(404, "Sorry，你输入的用户我们找不到！", ""));
            }
            user.MenuCode = request.MenuCode;
            user.MenuName = request.MenuName;
            var result= await _userManager.UpdateAsync(user);
            if (result.Succeeded)
            {
                return Ok(new ReturnTemplate(200, "配置成功啦！", ""));
            }
            else
            {
                return StatusCode(500, new ReturnTemplate(500, "删除用户时发生错误，原因请看ErrorResult", result.Errors));
            }


        }
    [HttpGet("role/all")]
    public async Task<IActionResult> AllRole()
    {
        
        return Ok(new ReturnTemplate(200,"查询成功啦！", _roleManager.Roles.ToList()));

    }
    
    

    /// <summary>
    /// 通过角色查询用户，支持分页
    /// </summary>
    /// <param name="request">包含角色名称、页码和每页大小的请求对象</param>
    /// <returns>返回包含总用户数和当前页用户的响应对象</returns>
    [HttpPost("user")]
    public async Task<IActionResult> SearchUserFromRole([FromBody] SearchUserFromRoleRequest request)
    {
       

        

        IList<UserProfile> usersInRole = null;
        if (string.IsNullOrWhiteSpace(request.RoleName))
        {
            usersInRole = await _userService.GetUsersProfileByUserNameAsync(request.UserName);
        }
        else
        {
           var usersInRole1 = await _userService.GetUsersProfileInRoleAsync(request.RoleName);
           usersInRole = usersInRole1.Where(a => a.UserName.Contains(request.UserName)).ToList();

        }

        var totalUsers = usersInRole.Count;

        var pagedUsers = usersInRole
            .Skip((request.Page - 1) * request.PageSize)
            .Take(request.PageSize)
            .ToList();

        var response = new SearchUserFromRoleResponse
        {
            TotalCount = totalUsers,
            Users = pagedUsers
        };

        return Ok(new ReturnTemplate(200, "查询成功", response));
    }

    /// <summary>
    /// 用于通过角色名称查询用户列表的请求模型。支持分页功能。
    /// </summary>
    public class SearchUserFromRoleRequest
    {
        
     
        public string UserName { get; set; }
        /// <summary>
        /// 表示角色的名称。此属性用于指定或获取与用户管理相关的角色名称。
        /// 在进行角色分配、查询等操作时，需要提供正确的角色名称以确保操作的成功执行。
        /// </summary>
        public string RoleName { get; set; }

        /// <summary>
        /// 表示当前请求的页码，默认为1。用于分页查询用户时指定从哪一页开始获取数据。
        /// </summary>
        public int Page { get; set; } = 1;

        /// <summary>
        /// 每页显示的用户数量。默认值为10。
        /// 该属性用于分页查询中指定每一页应包含的用户条目数。
        /// </summary>
        public int PageSize { get; set; } = 10;
    }

    /// <summary>
    /// 表示通过角色查询用户后返回的响应数据。
    /// 该类用于封装查询结果，包括总用户数和分页后的用户列表。
    /// </summary>
    public class SearchUserFromRoleResponse
    {
        /// <summary>
        /// 表示属于特定角色的用户总数。
        /// </summary>
        /// <remarks>此属性用于分页查询中，返回匹配给定角色名称的所有用户的数量。</remarks>
        public int TotalCount { get; set; }

        /// <summary>
        /// 表示属于特定角色的用户列表。该属性用于存储和返回在给定角色下的所有用户。
        /// </summary>
        /// <remarks>此列表通常作为查询结果的一部分，例如通过角色名搜索用户时返回的数据。</remarks>
        public List<UserProfile> Users { get; set; }
    }
}